/***************************************************************************
                       gvlistener.cpp  -  description
                             -------------------
    begin                : Son Jul 27 2003
    copyright            : (C) 2003 by Thomas Thiel
    email                : kv@wolfchild.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gvlistener.h"
#include <iostream>
#include <fstream>
#include <qevent.h>
#include <kapplication.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>


#define GV_ALL  589841
#define GV_TEAM 589842

#define GV_G1 589843
#define GV_G2 589844
#define GV_G3 589845
#define GV_G4 589846

#define GV_CMD  589847
#define GV_MUTE 589848

#define GV_ON  1
#define GV_OFF 0


GVListener::GVListener() : QThread()
{
  partner=NULL;
  hid_filename = "/dev/usb/hiddev0";
  hid_fd=-1;
}

 /** No descriptions */
 GVListener::GVListener(QObject *target, GVListenerStatus *status):QThread()
{
  gvls=status;
  partner=target;
  hid_filename = "/dev/usb/hiddev0";
  hid_fd=-1;
}

GVListener::~GVListener()
{

}


/** Thread Program - Here happens all the magic */
void GVListener::run()
{
  GVEvent *event;
  bool found = false;
  QString hiddev = "/dev/usb/hiddev";
  int cnt = 0;

  fd_set rfds;
  struct timeval tv;

  if (gvls==NULL)
  {
    std::cerr << "GV_Listener::run(): No GVListenerStatus issued - can't talk to main app...\n" << std::endl;
    return;
  };

  // lets assume everything's all right. If not, errors reset setRunning to false later on.
  gvls->setRunning(true);

  if (partner==NULL)
  {
    std::cerr << "GV_Listener::run(): No partner issued" << std::endl;
    gvls->setRunning(false);
    return;
  };

  // TODO: Insert Hid-Device Analyzer here.

  std::cout << "Starting to search for GameVoice via hiddev Module..." << std::endl;

  do
  {
    if (!((hid_fd = open ((hiddev+QString::number(cnt)), O_RDONLY)) < 0))
    {
      /* suck out some device information */
      ioctl (hid_fd, HIDIOCGDEVINFO, &device_info);

      /* the HIDIOCGDEVINFO ioctl() returns hiddev_devinfo
       * structure - see <linux/hiddev.h>
       * We look for vendor 0x045e product 0x003b - that's GV
       * (decimal vendor 1118 product 59)
       */
      if ((device_info.vendor==1118)&(device_info.product==59))
      {
        std::cout << "..." << hiddev+QString::number(cnt) << " exists, opens and IS actually a GameVoice." << std::endl;
        found = true;
      }
      else
      {
        std::cout << "..." << hiddev+QString::number(cnt) << " exists, opens and - unfortunately - is NO GameVoice." << endl;
        close(hid_fd);
      }
    }
    else
      std::cout << "..." << hiddev+QString::number(cnt) << " didn't open at all." << std::endl;

     // we checked enough devices, quit now.
    if (cnt>=16) found = true;

    cnt++;

  } while (!found);

  if (cnt>=16)
  {
    std::cerr << "failed to open hid device - KameVoice wont work." << std::endl;
    gvls->setRunning(false);
    return;
  };

  while (!gvls->termination_requested())
    {

      FD_ZERO(&rfds);
      FD_SET(hid_fd,&rfds);

      tv.tv_sec=2;
      tv.tv_usec=0;

      if (select(hid_fd+1,&rfds,NULL,NULL,&tv))
      {
        read_bytes = read (hid_fd, &ev, sizeof (struct hiddev_event) );

        if (read_bytes < (int) sizeof (struct hiddev_event))
        {
          std::cerr << "hiddevtest: short read" << std::endl;
          gvls->setRunning(false);
          return;
        }


        if (ev.hid==GV_G1)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::group1, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::group1, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }

        if (ev.hid==GV_G2)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::group2, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::group2, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }

        if (ev.hid== GV_G3)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::group3, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::group3, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }

        if (ev.hid== GV_G4)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::group4, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::group4, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }


        if (ev.hid== GV_TEAM)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::team, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::team, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }

        if (ev.hid== GV_ALL)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::all, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::all, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }

        if (ev.hid== GV_CMD)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::command, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::command, GVEvent::off );
            KApplication::postEvent(partner,event);
          }
        }

        if (ev.hid== GV_MUTE)
        {
          if (ev.value==GV_ON)
          {
            event = new GVEvent ( GVEvent::mute, GVEvent::on );
            KApplication::postEvent(partner,event);
          }
          else
          {
            event = new GVEvent ( GVEvent::mute, GVEvent::off );
            KApplication::postEvent(partner,event);
          };
        }

      };

    };

  close (hid_fd);

  std::cout << "GVListener thread terminated normally." << std::endl;
  gvls->setRunning(false);

  return;


}
